/*******************************************************************************
*                                                                          
*                   OSA Technologies, Inc. 
*       This software is supplied under the terms of a license agreement or
*       nondisclosure agreement with OSA Technologies, Inc. and may not be
*       copied or disclosed except in accordance with the terms of that
*       agreement.
*
*       Copyright 2003-2004 OSA Technologies Inc., an Avocent Company, San Jose, USA.
*
*------------------------------------------------------------------------------
*
* PROJECT        IPMI
* LANGUAGE       ANSI C
* SUBSYSTEM
*
* 
* Author: Ray Huang
*                                                                   
*
* File Description
* ----------------
* This is the Linux IPMI driver shim that solves the Linux versioning problem
*
*******************************************************************************/
#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/spinlock.h>
#include <linux/delay.h>
#include <linux/errno.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/param.h>

#ifdef __32BIT__
typedef unsigned int	PTR_t;
#else
typedef unsigned long long 	PTR_t;
#endif

MODULE_LICENSE("Proprietary.  Copyright 2003-2005 OSA Technologies Inc., an Avocent Company, San Jose, USA.");
/*
 *	Data movement to and from the kernel
 */
int shim$copy_to_user (void  *to, void  *from, size_t n)
{
	return (copy_to_user (to, from, n));
}


int shim$copy_from_user (void  *to, void  *from, size_t n)
{
	return (copy_from_user (to, from, n));
}


/*
 *	Heap management
 */
void  *shim$kmalloc (size_t  size, int  type)
{
	return ((void *)kmalloc (size, type));
}

void shim$kfree (const void  *chunk)
{
	kfree (chunk);
}

/*
 *	Driver registration
 */
int shim$register_chrdev (unsigned int  major, const char  *name, struct file_operations  *fops)
{
	return (register_chrdev (major, name, fops));
}

int shim$unregister_chrdev (unsigned int  major, const char   *name)
{
	return (unregister_chrdev (major, name));
}


/*
 *	Kernel data management
 */
void shim$memcpy (void  *dst, const void  *src, size_t  size)
{
	memcpy (dst, src, size);
}

void __shim$memcpy (void  *dst, const void  *src, size_t  size)
{
	__memcpy (dst, src, size);
}

int shim$memcmp (const void  *src, const void  *dst, size_t  size)
{
	return (memcmp (src, dst, size));
}

void shim$memset(void *dst, int c, size_t size)
{
	memset(dst, c, size);
}
char *shim$strcpy (char  *dst, char  *src)
{
	return (strcpy (dst, src));
}

/*
 *	i/o region management
 */
int shim$check_region (PTR_t base, unsigned int size)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	return (check_region(base, size));
#else
	return (__check_region (&ioport_resource, base, size));
#endif
}

struct resource   *shim$request_region (PTR_t start, unsigned int n, const char *name)
{
	return (request_region(start, n, name));
}


void  shim$release_region (PTR_t  base, unsigned int  size)
{
	release_region (base, size);
}

/*
 *	i/o memory region management
 */
int shim$check_mem_region (PTR_t base, unsigned int size)
{
	return (check_mem_region (base, size));
}

void shim$request_mem_region (PTR_t start, unsigned int n, const char *name)
{
	request_mem_region(start, n, name);
}


void  shim$release_mem_region (PTR_t  base, unsigned int  size)
{
	release_mem_region (base, size);
}

/*
 *	i/o mapping
 */
void *shim$ioremap (PTR_t offset, unsigned int size)
{
	return ioremap(offset, size);
}

void shim$iounmap (void * addr)
{
	iounmap (addr);
}


/*
 *	i/o 
 */
void shim$outb (unsigned char  data, PTR_t port)
{ 
	outb (data, port);
}

unsigned char  shim$inb (PTR_t port)
{
	return (inb (port));
}

/*
 *	i/o memory
 */
void shim$writeb (unsigned char  data, PTR_t addr)
{ 
	writeb(data, addr);
}

unsigned char  shim$readb (PTR_t addr)
{
	return (readb (addr));
}

void  *shim$phys_to_virt (PTR_t address)
{
	return ((void *)phys_to_virt (address)); 
}

/*
 *	spin lock management
 */
void shim$spin_lock_irqsave(spinlock_t *lock, unsigned long flags)
{
	spin_lock_irqsave(lock, flags);
}

void shim$spin_unlock_irqrestore (spinlock_t *lock, unsigned long flags)
{
	spin_unlock_irqrestore(lock,flags);
}

/*
 *	spin lock management
 */
void shim$spin_lock(spinlock_t *lock)
{
	spin_lock(lock);
}

void shim$spin_unlock (spinlock_t *lock)
{
	spin_unlock(lock);
}

/*
 *	time management
 */
void shim$mdelay(int time)
{
	mdelay (time);
}

/*
 *	time management
 */
void shim$udelay(int time)
{
	udelay (time);
}

int shim$access_ok(int type, const void *addr, unsigned long size)
{
	return access_ok(type,addr, size);
}

/*
	Debug functions
*/
void shim$printk(const char * fmt,...)
{
	char buf[1024];
	va_list args;
	int len;
	
	memset(buf, 0, sizeof(buf));
	va_start(args, fmt);
  	len = vsnprintf(buf, sizeof(buf), fmt, args);
  	va_end(args);
	printk("%s", buf);
}

